
#include <Wire.h>
#include <OneWire.h>
#include <TimerOne.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 20, 4);
//#include <LiquidCrystal.h>
//LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

#define BUTTON_PIN 12
#define T_PRESS 1800
#define T_CLICK 2500
//#define BEEP_PIN 13
#define BEEP_PIN 2
//#define VALVE_PIN 2
#define VALVE_PIN 13
#define DS_PIN 11
#define BEEP_FREQ 250
#define BEEP_ON 1000
#define BEEP_OFF 1000

#define POWER_MODE 1 // 1 - паразитное, 0 - внешнее
#define DS_PERIOD 4000 // 1 сек
byte cntCRC = 0;

#define tempAlarmMin 70.0
#define tempAlarm1 95.0
#define tempAlarm2 99.9
#define VALVE_PERIOD 1000  // 0.25 сек




#define Razgon 4
#define RazgonEnd 5
#define NaSebya 6
#define NaSebyaEnd 7
#define ShimG 8
#define ShimGConfirmation 9
#define Golovy 10
#define ShimT 11
#define ShimTConfirmation 12
#define Telo 13
#define AutoConfirmation 14
#define Auto 15
#define ShimA 16
#define ShimAConfirmation 17
#define Tauto 18
#define TautoConfirmation 19
#define StartStop 20
#define OtborEnd 21
#define ShimSS 22
#define ShimSSConfirmation 23
#define TautoSS 24
#define TautoSSConfirmation 25


OneWire sensDs (DS_PIN);

// обслуживание сенсорной кнопки
boolean startButton = false;
byte stateButton = 0;
int counterButton = 0;
int counterButtonDelay = 700;

class Button {
  public:
    boolean buttonClick = false;
    byte buttonType; // 0 - Long type, 1 - Type, 2 -Double type
    int lastStateButton = 0;

    void scanState();

  private:
    int buttonCode;
    boolean buttonPress = false;
};

Button button1;

int buttonIn; // какой тип выбрать?
int dsCount;
int copydsCount;
boolean dsFlagReady;
byte dsBuf[9];
float temperature;
float tempRazg = 30.0;
float tempAlarm;
float tempAuto;
float tempAutoTmp;
// float newtempAuto;
float hist = 0.125;

// для клапана
byte valveOn = 0;
byte valveOnTmp = 0;
byte countOn = 0;
byte countOff = 0;
byte valveOff = 0;
byte valveOffTmp = 0;
unsigned int valveCount;
#define valveOnHeadMax 12
#define valveOnHeadMin 2
#define valveOnBodyMax 14
#define valveOnBodyMin 2
byte valveOnHead = 4;
byte valveOffHead = 56; // 1 сек открыто из 15
byte valveOnBody = 10;
byte valveOffBody = 6;  // 2.5 сек открыто из 4
byte modeValve = LOW; // LOW - клапан закрыт, HIGH - открыт.
//byte vRed = 0; // 0 - редактируем valveOn Головы, 1 - valveOn Тело
// 2 - 10 valveOff, 3 - 1 valveOff

boolean k; // для клавиатуры

// для бибикалки
byte beepOut;
unsigned int beepCount;
boolean beepFlagOn;

// для таймера
unsigned int timerCount; // счётчик до 1 сек.
byte timerMin;
byte timerSec;
boolean secondFlagOn;
#define timerNaSebya 25 // минут


// для старт-стопа
#define timerStop  8 // минут

byte jobState; // 0 - запрос Брага или СС
// 1 - подтверждение браги
// 2 - подтверждение СС
// 3 - ждём Т воды Брага (tempAlarmMin) или ждём Т стоп брага (tempAlarm1) или отключен
// контроль Т браги (tempAlarm2)
// 4 - работа с СС, режим "разогрев"
// 5 - задаём скважность клапана, 51 - подтверждаем или нет заданную скважность
// 6 - подтверждение включения управления клапаном.
// 7 - отбор голов

//byte addrDS[8]={0x28, 0xFF, 0x83, 0x73, 0xB5, 0x16, 0x05, 0x1D}; // - номер датчика температуры - Китай
byte addrDS[8] = {0x28, 0xF8, 0x2C, 0x63, 0x1E, 0x19, 0x1, 0xB2}; // СИВ


void setup() {
  lcd.init();            // ------------------------ i2c или нет
  lcd.backlight();
  //  lcd.begin(16, 2);
  pinMode (BUTTON_PIN, INPUT);
  Serial.begin(115200);
  beepOut = HIGH;
  pinMode(BEEP_PIN, OUTPUT);
  digitalWrite(BEEP_PIN, beepOut);
  beepCount = 0;
  Timer1.initialize(BEEP_FREQ);
  Timer1.attachInterrupt(beepOutput);
  dsCount = 0;
  dsFlagReady = true;
  tempAlarm = tempAlarmMin; // изменяется в ходе выполнения
  beepFlagOn = false;
  jobState = 0;
  menuState0 ();
}

void loop() {
  //  Serial.println(jobState);
  if (jobState == 0)
  {
    button1.scanState();
    if ((button1.buttonClick == true) & (button1.buttonType == 0)) {
      button1.buttonClick = false; jobState = 1;
      lcd.setCursor(0, 0); lcd.print("     BRAGA      ");
      lcd.setCursor(0, 1); lcd.print("Y-Long  N-Double");
    }
    if ((button1.buttonClick == true) & (button1.buttonType == 2)) {
      button1.buttonClick = false; jobState = 2;
      lcd.setCursor(0, 0); lcd.print("       SS       ");
      lcd.setCursor(0, 1); lcd.print("Y-Long  N-Double");
    }
  }

  if (jobState == 1)
  {
    button1.scanState();
    if ((button1.buttonClick == true) & (button1.buttonType == 2)) {
      button1.buttonClick = false; jobState = 0;
      menuState0 ();
    }
    if ((button1.buttonClick == true) & (button1.buttonType == 0)) {
      button1.buttonClick = false; jobState = 3;
      lcd.clear();
      lcd.setCursor(0, 0); lcd.print("Temp:     "); lcd.print(temperature);
      lcd.setCursor(0, 1); lcd.print("Water on: "); lcd.print(tempAlarm);
    }
  }

  if (jobState == 2)
  {
    button1.scanState();
    if ((button1.buttonClick == true) & (button1.buttonType == 2)) {
      jobState = 0; button1.buttonClick = false;
      menuState0 ();
    }
    if ((button1.buttonClick == true) & (button1.buttonType == 0)) {
      jobState = Razgon ; button1.buttonClick = false;
      menuRazgon ();
    }
  }

  if (jobState == 3)
  {
    lcd.setCursor(10, 0); lcd.print(temperature);
    button1.scanState();
    if ((button1.buttonClick == true) & (button1.buttonType == 1) & (tempAlarm == tempAlarmMin)) {
      tempAlarm = tempAlarm1;
      lcd.setCursor(0, 1); lcd.print("TempStop: "); lcd.print(tempAlarm);
      button1.buttonClick = false;
    }
    if ((button1.buttonClick == true) & (button1.buttonType == 1) & (tempAlarm == tempAlarm1)) {
      tempAlarm = tempAlarm2;
      lcd.setCursor(0, 1); lcd.print("StopBeep: "); lcd.print(tempAlarm);
      button1.buttonClick = false;
    }
    if ((button1.buttonClick == true) & (button1.buttonType == 1) & (tempAlarm == tempAlarm2)) {
      tempAlarm = tempAlarmMin;
      lcd.setCursor(0, 1); lcd.print("Water on: "); lcd.print(tempAlarm);
      button1.buttonClick = false;
    }
    lcd.setCursor(10, 1); lcd.print(tempAlarm);
    if (temperature < tempAlarm) {
      beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
    }
    if (temperature >=  tempAlarm) {
      beepFlagOn = true;
    }
  }

  if (jobState == Razgon)
  {
    button1.scanState();
    k0();
    if (k == true) {
      jobState = NaSebya ;
      timerMin = timerNaSebya; timerSec = 0;
      menuNaSebya ();
      secondFlagOn = false;
      timerCount = 0;
    }
    k1();
    if ((k == true)&(tempRazg >= 35.0)) {
      tempRazg = tempRazg - 5.0;
      lcd.setCursor(15, 1); lcd.print(tempRazg);
    }
    k2();
    if ((k == true)&(tempRazg <= 65.0)) {
      tempRazg = tempRazg + 5.0;
      lcd.setCursor(15, 1); lcd.print(tempRazg);
    }
    button1.buttonClick = false;
    
    lcd.setCursor(3, 1); lcd.print(temperature);
    if (temperature >= tempRazg) {
      jobState = RazgonEnd ; menuRazgonEnd ();
      beepFlagOn = true;
    }
  }

  if (jobState == RazgonEnd)
  {
    button1.scanState(); k0 ();
    if (k == true) {
      jobState = NaSebya ;
      timerMin = timerNaSebya; timerSec = 0;
      beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
      menuNaSebya ();
      secondFlagOn = false;
      timerCount = 0;
    }
    button1.buttonClick = false;
  }

  if (jobState == NaSebya)
  {
    lcd.setCursor(3, 1); lcd.print(temperature);

    if (secondFlagOn == true) {
      secondFlagOn = false;
      if (timerSec == 0) {
        timerSec = 59; timerMin--;
      }
      timerSec--;
      secondFlagOn = false;
    }

    lcd.setCursor(15, 1);
    if (timerMin < 10) {
      lcd.print("0");
    }
    lcd.print(timerMin); lcd.print(F(":"));
    if (timerSec < 10) {
      lcd.print("0");
    }
    lcd.print(timerSec);

    if ((timerMin == 0) & (timerSec == 0)) {
      //      timerFlagOn = false;
      menuNaSebyaEnd ();
      beepFlagOn = true; jobState = NaSebyaEnd;
    }

    button1.scanState();
    k0 ();
    if (k == true) {
      //      timerFlagOn = false;
      menuShimG ();
      jobState = ShimG;
    }

    k1 ();
    if ((k == true) & (timerMin >= (timerNaSebya))) {
      timerMin = timerMin - 5 ;
    }

    k2 ();
    if ((k == true) & (timerMin <= (timerNaSebya + 10))) {
      timerMin = timerMin + 5 ;
    }
    button1.buttonClick = false;
  }

  if (jobState == NaSebyaEnd) {
    button1.scanState(); k0 ();
    if (k == true) {
      beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
      valveOn = valveOnHead; valveOff = valveOffHead;
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimG ();
      jobState = ShimG;
    }
    button1.buttonClick = false;
  }

  if (jobState == ShimG) {
    button1.scanState();
    k1();
    if ((k == true) & (valveOn > valveOnHeadMin)) {
      valveOn-- ; valveOff++ ;
    }
    k2();
    if ((k == true) & (valveOn < valveOnHeadMax)) {
      valveOn++ ; valveOff-- ;
    }
    k0();
    if (k == true) {
      menuShimGConfirmation (); jobState = ShimGConfirmation;
    }

    lcd.setCursor(7, 1);
    if (valveOn < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOn);

    lcd.setCursor(10, 1);
    if (valveOff < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOff);

    button1.buttonClick = false;
  }

  if (jobState == ShimGConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      menuGolovy(); countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      jobState = Golovy;
    }
    k2();
    if (k == true) {
      valveOn = valveOnTmp; valveOff = valveOffTmp;
      menuShimG(); jobState = ShimG;
    }
    button1.buttonClick = false;
  }

  if (jobState == Golovy) {
    lcd.setCursor(3, 1); lcd.print(temperature);
    if (valveCount >= VALVE_PERIOD) {
      valveCount = 0;
      if (modeValve == HIGH) {
        countOn--;
        if (countOn == 0) {
          modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
          countOn = 0; countOff = valveOff;
        }
      }
      else {
        countOff--;
        if (countOff == 0) {
          modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
          countOn = valveOn; countOff = 0;
        }
      }
      lcd.setCursor(15, 1);
      if (countOn < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOn); lcd.print(F("/"));
      if (countOff < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOff);
    }
    button1.scanState(); k0();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
      valveOn = valveOnBody; valveOff = valveOffBody;
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimT ();
      jobState = ShimT;
      button1.buttonClick = false;
    }
    k2();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimG ();
      jobState = ShimG;
      button1.buttonClick = false;
    }
  }

  if (jobState == ShimT) {
    button1.scanState();
    k1();
    if ((k == true) & (valveOn > valveOnBodyMin)) {
      valveOn-- ; valveOff++ ;
    }
    k2();
    if ((k == true) & (valveOn < valveOnBodyMax)) {
      valveOn++ ; valveOff-- ;
    }
    k0();
    if (k == true) {
      menuShimTConfirmation (); jobState = ShimTConfirmation;
    }

    lcd.setCursor(7, 1);
    if (valveOn < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOn);

    lcd.setCursor(10, 1);
    if (valveOff < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOff);

    button1.buttonClick = false;
  }

  if (jobState == ShimTConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      menuTelo(); countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      jobState = Telo;
    }
    k2();
    if (k == true) {
      valveOn = valveOnTmp; valveOff = valveOffTmp;
      menuShimT(); jobState = ShimT;
    }
    button1.buttonClick = false;
  }

  if (jobState == Telo) {
    lcd.setCursor(3, 1); lcd.print(temperature);
    if (valveCount >= VALVE_PERIOD) {
      valveCount = 0;
      if (modeValve == HIGH) {
        countOn--;
        if (countOn == 0) {
          modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
          countOn = 0; countOff = valveOff;
        }
      }
      else {
        countOff--;
        if (countOff == 0) {
          modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
          countOn = valveOn; countOff = 0;
        }
      }
      lcd.setCursor(15, 1);
      if (countOn < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOn); lcd.print(F("/"));
      if (countOff < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOff);
    }
    button1.scanState(); k2();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
      valveOn = valveOnBody; valveOff = valveOffBody;
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimT ();
      jobState = ShimT;
      button1.buttonClick = false;
    }
    k0();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. AUTO
      //      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuAutoConfirmation ();
      jobState = AutoConfirmation;
      button1.buttonClick = false;
    }
  }

  if (jobState == AutoConfirmation) {
    button1.scanState(); k2();
    if (k == true) {
      menuTelo(); countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      jobState = Telo;
      button1.buttonClick = false;
    }
    k0();
    if (k == true) {
      menuAuto (); countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      tempAuto = temperature;
      jobState = Auto;
      button1.buttonClick = false;
    }
  }

  if (jobState == Auto) {
    lcd.setCursor(2, 1); lcd.print(temperature);
    lcd.setCursor(15, 1); lcd.print(tempAuto);
    if (valveCount >= VALVE_PERIOD) {
      valveCount = 0;
      if (modeValve == HIGH) {
        countOn--;
        if (countOn == 0) {
          modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
          countOn = 0; countOff = valveOff;
        }
      }
      else {
        countOff--;
        if (countOff == 0) {
          modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
          countOn = valveOn; countOff = 0;
        }
      }
      lcd.setCursor(15, 0);
      if (countOn < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOn); lcd.print(F("/"));
      if (countOff < 10) {
        lcd.print(F("0"));
      }
      lcd.print(countOff);
    }
    button1.scanState(); k2();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimA ();
      jobState = ShimA;
      button1.buttonClick = false;
    }
    k0();
    if (k == true) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. Tauto
      tempAutoTmp = tempAuto;
      menuTauto ();
      jobState = Tauto;
      button1.buttonClick = false;
    }
    if (temperature >= (tempAuto + hist)) {
      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время START/STOP
      menuStartStop ();
      timerMin = timerStop; timerSec = 0;
      beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
      secondFlagOn = false;
      timerCount = 0;
      jobState = StartStop;
    }
  }

  if (jobState == ShimA) {
    button1.scanState();
    k1();
    if ((k == true) & (valveOn > valveOnBodyMin)) {
      valveOn-- ; valveOff++ ;
    }
    k2();
    if ((k == true) & (valveOn < valveOnBodyMax)) {
      valveOn++ ; valveOff-- ;
    }
    k0();
    if (k == true) {
      menuShimAConfirmation (); jobState = ShimAConfirmation;
    }

    lcd.setCursor(7, 1);
    if (valveOn < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOn);

    lcd.setCursor(10, 1);
    if (valveOff < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOff);

    button1.buttonClick = false;
  }

  if (jobState == ShimAConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      menuAuto(); countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      jobState = Auto;
    }
    k2();
    if (k == true) {
      valveOn = valveOnTmp; valveOff = valveOffTmp;
      menuShimA(); jobState = ShimA;
    }
    button1.buttonClick = false;
  }

  if (jobState == Tauto) {
    button1.scanState();
    k1();
    if ((k == true) & (tempAuto > tempAutoTmp)) {
      tempAuto = tempAuto - 0.0625 ;
    }
    k2();
    if ((k == true) & (tempAuto < (tempAutoTmp + 0.25))) {
      tempAuto = tempAuto + 0.0625 ;
    }
    k0();
    if (k == true) {
      menuTautoConfirmation (); jobState = TautoConfirmation;
    }
    lcd.setCursor(8, 1); lcd.print(tempAuto);
    button1.buttonClick = false;
  }

  if (jobState == TautoConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      menuAuto(); jobState = Auto;
    }
    k2();
    if (k == true) {
      tempAuto = tempAutoTmp;
      menuTauto(); jobState = Tauto;
    }
    button1.buttonClick = false;
  }

  if (jobState == StartStop) {
    lcd.setCursor(4, 1); lcd.print(temperature);

    if (secondFlagOn == true) {
      secondFlagOn = false;
      if (timerSec == 0) {
        timerSec = 59; timerMin--;
      }
      timerSec--;
      secondFlagOn = false;
    }

    lcd.setCursor(13, 1);
    if (timerMin < 10) {
      lcd.print("0");
    }
    lcd.print(timerMin); lcd.print(F(":"));
    if (timerSec < 10) {
      lcd.print("0");
    }
    lcd.print(timerSec);

    if ((timerMin == 0) & (timerSec == 0)) {
      //      timerFlagOn = false;
      menuOtborEnd ();
      beepFlagOn = true; jobState = OtborEnd;;
    }

    button1.scanState(); k2();
    if (k == true) {
      //      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
      valveOnTmp = valveOn; valveOffTmp = valveOff;
      menuShimA ();
      jobState = ShimSS;
      button1.buttonClick = false;
    }
    k0();
    if (k == true) {
      //      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. Tauto
      tempAutoTmp = tempAuto;
      menuTauto ();
      jobState = TautoSS;
      button1.buttonClick = false;
    }
    if (temperature <= tempAuto) {
      countOn = valveOn; countOff = valveOff;
      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      menuAuto(); jobState = Auto;
    }
  }

  if (jobState == ShimSS) {
    button1.scanState();
    k1();
    if ((k == true) & (valveOn > valveOnBodyMin)) {
      valveOn-- ; valveOff++ ;
    }
    k2();
    if ((k == true) & (valveOn < valveOnBodyMax)) {
      valveOn++ ; valveOff-- ;
    }
    k0();
    if (k == true) {
      menuShimAConfirmation (); jobState = ShimSSConfirmation;
    }

    lcd.setCursor(7, 1);
    if (valveOn < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOn);

    lcd.setCursor(10, 1);
    if (valveOff < 10) {
      lcd.print(F("0"));
    }
    lcd.print(valveOff);

    button1.buttonClick = false;
  }

  if (jobState == ShimSSConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      //      menuAuto(); countOn = valveOn; countOff = valveOff;
      //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      menuStartStop(); jobState = StartStop;
    }
    k2();
    if (k == true) {
      valveOn = valveOnTmp; valveOff = valveOffTmp;
      menuShimA(); jobState = ShimSS;
    }
    button1.buttonClick = false;
  }

  if (jobState == TautoSS) {
    button1.scanState();
    k1();
    if ((k == true) & (tempAuto > tempAutoTmp)) {
      tempAuto = tempAuto - 0.0625 ;
    }
    k2();
    if ((k == true) & (tempAuto < (tempAutoTmp + 0.25))) {
      tempAuto = tempAuto + 0.0625 ;
    }
    k0();
    if (k == true) {
      menuTautoConfirmation (); jobState = TautoSSConfirmation;
    }
    lcd.setCursor(8, 1); lcd.print(tempAuto);
    button1.buttonClick = false;
  }

  if (jobState == TautoSSConfirmation) {
    button1.scanState();
    k0();
    if (k == true) {
      //      countOn = valveOn; countOff = valveOff;
      //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      menuStartStop(); jobState = StartStop;
    }
    k2();
    if (k == true) {
      tempAuto = tempAutoTmp;
      menuTauto(); jobState = TautoSS;
    }
    button1.buttonClick = false;
  }



  if (jobState == OtborEnd) {
    button1.scanState(); k0 ();
    if (k == true) {
      beepFlagOn = false;  digitalWrite(BEEP_PIN, HIGH);
      beepOut = HIGH;
      digitalWrite(BEEP_PIN, beepOut);
    }
    button1.buttonClick = false;
  }

  // работа с термодатчиком
  if (dsFlagReady == true) {
    sensDs.reset();
    sensDs.write(0xCC, POWER_MODE);
    sensDs.write(0x44, POWER_MODE); //запуск измерения
    dsCount = 0; dsFlagReady = false;
  }

  if (copydsCount >= DS_PERIOD) {
    sensDs.reset();
    sensDs.select(addrDS);
    //    sensDs.write(0xCC, POWER_MODE);
    sensDs.write(0xBE, POWER_MODE); //команда чтения
    sensDs.read_bytes(dsBuf, 9);
    if (OneWire::crc8(dsBuf, 8) == dsBuf[8]) {
      temperature = (float)((int)dsBuf[0] | (((int)dsBuf[1]) << 8)) * 0.0625 + 0.3125;
//      Serial.println("ok");
    Serial.println(temperature);
    }
    else {
      Serial.println("CRC error"); cntCRC++; lcd.setCursor(5, 0); lcd.print("CRC"); lcd.print(cntCRC);
    }
    dsCount = 0; dsFlagReady = true;
  }
} // -------------- закрываем loop ----------------------


void beepOutput ()
{
  if (beepFlagOn == true) {
    beepCount++;
    if (beepCount < BEEP_ON) {
      digitalWrite(BEEP_PIN, beepOut);
      beepOut = !beepOut;
    }
    if (beepCount == BEEP_ON) {
      beepOut = HIGH;
      digitalWrite(BEEP_PIN, beepOut);
    }
    if (beepCount >= BEEP_ON + BEEP_OFF) {
      beepCount = 0;
    }
  }
  // работа с термодатчиком
  dsCount++;
  copydsCount = dsCount;

  // работа с клапаном
  valveCount++;

  // работа с сенсорной кнопкой
  if (startButton == true) {
    counterButton++;
  }

  //работа с таймером
  timerCount++;
  if (timerCount == 4000) {
    timerCount = 0;
    secondFlagOn = true;
  }
}

// обработчик кнопки --------------------------------
void Button::scanState() {
  buttonIn = digitalRead(BUTTON_PIN);
  // состояние 0-----------------------------------------------------------------
  if (stateButton == 0)
  {
    if ((lastStateButton == 0) & (buttonIn == 1))
    {
      lastStateButton = 1; stateButton = 1; startButton = false; counterButton = 0; startButton = true;
    }
  }
  // состояние 1 ----------------------------------------------------------------
  if (stateButton == 1)
  {

    if ((lastStateButton == 1) & (buttonIn == 0) & (counterButton < T_PRESS))
    {
      //      Serial.println (counterButton);
      lastStateButton = 0; stateButton = 2; startButton = false; counterButton = 0; startButton = true;
    }
    if ((lastStateButton == 1) & (buttonIn == 1) & (counterButton >= T_PRESS))
    {
      //      buttonClick=true ;
      stateButton = 6; startButton = false; counterButton = 0;
    }
  }
  // состояние 6 (длительное нажатие) ------------------------------------------
  if (stateButton == 6)
  {
    if ((lastStateButton == 1) & (buttonIn == 0))
    {
      lastStateButton = 0; buttonType = 0; startButton = false; counterButton = 0; stateButton = 0; buttonClick = true ;
    }

  }
  // состояние 2 (ждём будет второй клик или нет) ------------------------------
  if (stateButton == 2)
  {
    if ((lastStateButton == 0) & (buttonIn == 1) & (counterButton < T_CLICK))
    {
      lastStateButton = 1; stateButton = 3; startButton = false; counterButton = 0;
    }
    if ((lastStateButton == 0) & (buttonIn == 0) & (counterButton >= T_CLICK))
    {
      buttonClick = true; buttonType = 1; startButton = false; counterButton = 0; stateButton = 0;
    }
  }
  // состояние 3 (двойное нажатие кнопки) --------------------------------------
  if (stateButton == 3)
  {
    if ((lastStateButton == 1) & (buttonIn == 0))
    {
      buttonClick = true; lastStateButton = 0; buttonType = 2; stateButton = 0;
    }
  }
} // закрываем обработчик кнопки

void k0 () {
  if ((button1.buttonClick == true) & (button1.buttonType == 0)) {
    k = true;
  }
  else {
    k = false;
  }
}

void k1 () {
  if ((button1.buttonClick == true) & (button1.buttonType == 1)) {
    k = true;
  }
  else {
    k = false;
  }
}

void k2 () {
  if ((button1.buttonClick == true) & (button1.buttonType == 2)) {
    k = true;
  }
  else {
    k = false;
  }
}



void menuState0 () {
  lcd.setCursor(0, 0); lcd.print("BRAGA-Long press");
  lcd.setCursor(0, 1); lcd.print("SS-Double click ");
}

void menuRazgon () {
  lcd.setCursor(0, 0); lcd.print(F("       RAZGON       "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); 
  lcd.print(F("   Tr: "));  lcd.print(tempRazg); 
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("  LONG -  NA SEBYA  "));
}

void menuRazgonEnd () {
  lcd.setCursor(0, 0); lcd.print(F("  RAZGON ZAKONCHEN  "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("  LONG -  NA SEBYA  "));
}

void menuNaSebya () {
  lcd.setCursor(0, 0); lcd.print(F("      NA SEBYA      "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F(" TIMER: "));
  lcd.print(timerMin); lcd.print(":00");
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("   LONG -  GOLOVY   "));
}

void menuNaSebyaEnd () {
  lcd.setCursor(0, 0); lcd.print(F(" NA SEBYA ZAKONCHEN "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("   LONG -  GOLOVY   "));
}

void menuShimG () {
  valveOn = valveOnHead; valveOff = valveOffHead;
  lcd.setCursor(0, 0); lcd.print(F("     SHIM GOLOVY    "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}

void menuShimGConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("     SHIM GOLOVY    "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOn); lcd.print(F("/"));
  if (valveOff < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}

void menuGolovy () {
  lcd.setCursor(0, 0); lcd.print(F("       GOLOVY       "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F("  "));
  lcd.print(F("SHIM:"));
  if (valveOn < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOn); lcd.print(F("/"));
  if (valveOff < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - TELO     "));
}

void menuShimT () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM TELO     "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}

void menuShimTConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM TELO     "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}

void menuTelo () {
  lcd.setCursor(0, 0); lcd.print(F("        TELO        "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F("  "));
  lcd.print(F("SHIM:"));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - AUTO     "));
}

void menuAutoConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("        AUTO        "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}

void menuAuto () {
  lcd.setCursor(0, 0);
  lcd.print(F("AUTO     SHIM: "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 1);
  lcd.print(F("T:")); lcd.print(temperature);
  lcd.print(F("  Tauto:")); lcd.print(tempAuto);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - Tauto    "));
}

void menuStartStop () {
  lcd.setCursor(0, 0);
  lcd.print(F("     ")); lcd.print(F("Start/Stop")); lcd.print(F("     "));
  lcd.setCursor(0, 1);
  lcd.print(F("  ")); lcd.print(F("T:")); lcd.print(temperature); lcd.print(F("    "));
  lcd.print(F("00:00")); lcd.print(F("  "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - Tauto    "));
}

void menuShimA () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM AUTO     "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}

void menuShimAConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM AUTO     "));
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}

void menuTauto () {
  lcd.setCursor(0, 0); lcd.print(F("        Tauto       "));
  lcd.setCursor(0, 1); lcd.print(F("        ")); lcd.print(temperature); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}

void menuTautoConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("        Tauto       "));
  lcd.setCursor(0, 1); lcd.print(F("        ")); lcd.print(temperature); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}

void menuOtborEnd () {
  lcd.setCursor(0, 0); lcd.print(F("                    "));
  lcd.setCursor(0, 1); lcd.print(F("     ")); lcd.print(F("OTBOR STOP")); lcd.print(F("     "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("                    "));
}
